#include <Joystick.h>

Joystick_ controller(
	JOYSTICK_DEFAULT_REPORT_ID, 
	JOYSTICK_TYPE_GAMEPAD, 
	0,		// buttonCount,
	0,		// hatSwitchCount
	true,	// XAxis,
	true,	// YAxis,
	true,	// ZAxis,
	true,	// RxAxis,
	true,	// RyAxis,
	true,	// RzAxis,
	true,	// Rudder,
	false,	// Throttle
	false,	// Accelerator
	false,	// Brake,
	false	// Steering
);

#define CHNUM 7
const int pin[CHNUM] = { 4, 13, 0, 1, 2, 3, 7};
struct pulse {
	int32_t min;
	int32_t max;
} pulse[CHNUM] = {
	{ 1070, 1970},
	{ 1070, 1970},
	{ 1070, 1970},
	{ 1070, 1970},
	{ 1070, 1970},
	{ 1070, 1970},
	{ 1070, 1970},
};
volatile unsigned long t[CHNUM], w[CHNUM];
unsigned int last[CHNUM];

/*
https://forum.arduino.cc/t/pwm-input-capture-interrupt/544530
*/
ISR(TIMER1_CAPT_vect)
{
	static volatile unsigned int t0 = 0;
	static volatile unsigned int t1 = 0;

	if (TCCR1B & (1 << ICES1))
	{
		t0 = ICR1;
		TCCR1B &= ~(1 << ICES1);
	}
	else
	{
		t1 = ICR1;
		TCCR1B |= (1 << ICES1);
		w[0] = (t1 - t0) * 62.5 / 1000;
	}
}

ISR(TIMER3_CAPT_vect)
{
	static volatile unsigned int t0 = 0;
	static volatile unsigned int t1 = 0;

	if (TCCR3B & (1 << ICES3))
	{
		t0 = ICR3;
		TCCR3B &= ~(1 << ICES3);
	}
	else
	{
		t1 = ICR3;
		TCCR3B |= (1 << ICES3);
		w[1] = (t1 - t0) * 62.5 / 1000;
	}
}

void PwmGet(int ch){
	if(digitalRead(pin[ch])){
		t[ch] = micros();
	}else{
		w[ch] = micros() - t[ch];
	}
}

void PwmGet2(){PwmGet(2);}
void PwmGet3(){PwmGet(3);}
void PwmGet4(){PwmGet(4);}
void PwmGet5(){PwmGet(5);}
void PwmGet6(){PwmGet(6);}

void setup()
{
	for(int i = 0; i <= 16; i++){
		pinMode(i, INPUT_PULLUP);
	}

	pinMode(A0, INPUT_PULLUP);
	pinMode(A1, INPUT_PULLUP);
	pinMode(A2, INPUT_PULLUP);
	pinMode(A3, INPUT_PULLUP);
	pinMode(A4, INPUT_PULLUP);

	pinMode(A5, OUTPUT);
	digitalWrite(A5, LOW); 

	Serial.begin(115200);

	for(int i = 0; i < CHNUM; i++){
		w[i] = 0;
		last[i] = 0;
	}

	noInterrupts ();
	TCCR1A = 0;
	TCCR1B = 0;
	TCNT1  = 0;
	TIFR1  |= (1<<ICF1);
	TCCR1B |= (1<<CS10);
	TCCR1B |= (1<<ICES1);
	TIMSK1 |= (1<<ICIE1);

	TCCR3A = 0;
	TCCR3B = 0;
	TCNT3  = 0;
	TIFR3  |= (1<<ICF3);
	TCCR3B |= (1<<CS30);
	TCCR3B |= (1<<ICES3);
	TIMSK3 |= (1<<ICIE3);

	attachInterrupt(digitalPinToInterrupt(pin[2]), PwmGet2, CHANGE);
	attachInterrupt(digitalPinToInterrupt(pin[3]), PwmGet3, CHANGE);
	attachInterrupt(digitalPinToInterrupt(pin[4]), PwmGet4, CHANGE);
	attachInterrupt(digitalPinToInterrupt(pin[5]), PwmGet5, CHANGE);
	attachInterrupt(digitalPinToInterrupt(pin[6]), PwmGet6, CHANGE);

	controller.setXAxisRange(pulse[0].min, pulse[0].max);
	controller.setYAxisRange(pulse[1].min, pulse[1].max);
	controller.setZAxisRange(pulse[2].min, pulse[2].max);
	controller.setRxAxisRange(pulse[3].min, pulse[3].max);
	controller.setRyAxisRange(pulse[4].min, pulse[4].max);
	controller.setRzAxisRange(pulse[5].min, pulse[5].max);
	controller.setRudderRange(pulse[6].min, pulse[6].max);

	controller.begin(false);
	interrupts ();
}

void loop()
{
	bool sendUpdate = false;

	for(int i = 0; i < CHNUM; i++){
		unsigned int current = w[i];
		if (current != last[i]){
			last[i] = current;

			switch(i){
				case 0:	controller.setXAxis(current);break;
				case 1:	controller.setYAxis(current);break;
				case 2:	controller.setZAxis(current);break;
				case 3:	controller.setRxAxis(current);break;
				case 4:	controller.setRyAxis(current);break;
				case 5:	controller.setRzAxis(current);break;
				case 6:	controller.setRudder(current);break;
			}
			sendUpdate = true;
		}
	}

	if(sendUpdate)
	{
		controller.sendState();
	}

	delay(1);

	if(digitalRead(A4) == LOW)
	{
		for(int ch = 0; ch < CHNUM; ch++){
			Serial.print(w[ch]);
			Serial.print(',');
		}
		Serial.print('\n');
	}
}
